startEpiviz <- function(port=7312L, localURL=NULL, useDevel=FALSE, 
                        chr="chr11", start=99800000, end=103383180, 
                        debug=FALSE, proxy=TRUE, workspace=NULL, 
                        openBrowser=TRUE,
                        verbose=FALSE, nonInteractive=FALSE, tryPorts=FALSE) {
  if (verbose) {
    message("Starting Epivizr!")
  }

  server <- epivizr:::EpivizServer$new(port=port, tryPorts=tryPorts)
  
  if (missing(localURL) || is.null(localURL)) {
    url <- ifelse(useDevel,"epiviz-dev", "epiviz")
    url <- sprintf("http://%s.cbcb.umd.edu/index.php", url)
  } else {
    url <- localURL
  }
  
  wsURL <- "ws://localhost"
  controllerHost <- sprintf("%s:%d", wsURL, port)
  
  url <- sprintf("%s?controllerHost=%s&debug=%s&proxy=%s&", 
              url,
              controllerHost,
              ifelse(debug,"true","false"),
              ifelse(proxy,"true","false"))
  
  if (!is.null(workspace)) {
    url <- paste0(url,"workspace=",workspace,"&")
  } else {
    url <- paste0(url,
               sprintf("chr=%s&start=%d&end=%d&",
                       chr,
                       as.integer(start),
                       as.integer(end)))
  }

  if (verbose) {
    message("Initializing session manager...")
  }
  tryCatch({
    mgr <- EpivizDeviceMgr$new(server=server, url=url, verbose=verbose, nonInteractive=nonInteractive)
    mgr$bindToServer()
  }, error=function(e) {
    server$stopServer()
    stop("Error starting Epiviz: ", e)
  })
  
  if (verbose) {
    message("Opening connections...")
  }

  if (openBrowser) {
    tryCatch({
      mgr$openBrowser(url)
    }, error=function(e) {
      mgr$stopServer()
      stop("Error starting Epiviz: ", e)
    }, interrupt=function(e) {NULL})
  }

  if (verbose) {
    message("Done starting Epivizr!")
  }
  return(mgr)
}
